// Buffer.h

#ifndef Buffer_h
#define Buffer_h

#ifndef Data_h
#include "Data.h"
#endif
#ifndef ConstData_h
#include "ConstData.h"
#endif

class ConstBuffer;

/*
	A Buffer is like a Data which shrinks (from the front) as you
	fill it up.  You don't create Buffers, but rather BufferMasters.
	That gives you special powers, like seeing the data that's been
	put inside.  Then you pass the Buffer& to functions, which can
	fill it up.
*/
	
class Buffer
  {
	friend class BufferMaster;
	friend Buffer& operator<<( Buffer&, ConstData );
	
	private:
		Data unused;
	
		explicit Buffer( Data space )
		  : unused( space )
		  {}

		~Buffer()		{}
		
		// not implemented:
			Buffer( const Buffer& );
			void operator=( const Buffer& );
	
	public:
		uint8& operator[]( uint32 i )						{ return unused[i]; }
		
		Data Unused()											{ return unused; }
		ConstData Unused() const							{ return unused; }
		operator Data()										{ return unused; }

		uint8 *UnusedStart()									{ return unused.Start(); }
		uint8 *UnusedEnd()									{ return unused.End(); }

		const uint8 *UnusedStart() const					{ return unused.Start(); }
		const uint8 *UnusedEnd() const					{ return unused.End(); }
		uint32 UnusedLength() const						{ return unused.Length(); }

		void AdvanceMark( uint32 amount )				{ unused = unused.Tail( amount ); }

		bool Exhausted() const								{ return unused.IsEmpty(); }

		void Put( uint8 c )									{ unused[0] = c; unused = unused.Tail(1); }
  };

inline Buffer& operator<<( Buffer& to, ConstData from )
  {
	to.unused = to.unused << from;
	return to;
  }

Buffer& operator<<( Buffer& to, ConstBuffer& from );
	// returns to, for chaining

#endif
