// ApplicationWaker.cp

#ifndef ApplicationWaker_h
#include "ApplicationWaker.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef DeferredTaskTime_h
#include "DeferredTaskTime.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif

ApplicationWaker::ApplicationWaker()
  : awake( true ),
	 ringing( false ),
	 canSleep( true )
  {
	Assert( !DeferredTaskTime::IsNow() );
	
	DebugOSError( GetCurrentProcess( &process ) );
	
	#if GENERATINGCFM
		static RoutineDescriptor RingAlarmGlue =
			BUILD_ROUTINE_DESCRIPTOR( uppTimerProcInfo, RingAlarm );
	#endif

	timeTask.tmAddr = &RingAlarmGlue;
  }

ApplicationWaker& ApplicationWaker::The()
  {
	static ApplicationWaker theWaker;
	return theWaker;
  }

void ApplicationWaker::StartAlarm()
  {
	canSleep = false;
	
	if ( awake || ringing )
		return;
	
	ringing = true;
	DebugOSError( WakeUpProcess( &process ) );
	timeTask.tmWakeUp = 0;
	InsTime( reinterpret_cast<QElem *>( &timeTask ) );
	PrimeTime( reinterpret_cast<QElem *>( &timeTask ), 10 );
  }

void ApplicationWaker::StopAlarm()
  {
	awake = true;
	
	if ( ringing )
	  {
		RmvTime( reinterpret_cast<QElem *>( &timeTask ) );
		ringing = false;
	  }

	canSleep = true;
  }

bool ApplicationWaker::CanSleep()
  {
	Assert( awake );
	Assert( !ringing );
	
	awake = !canSleep;
	return canSleep;
  }

pascal void ApplicationWaker::RingAlarm( ApplicationWaker *self )
  {
	if ( self->awake )
		return;
	
	DebugOSError( WakeUpProcess( &self->process ) );
	PrimeTime( reinterpret_cast<QElem *>( &self->timeTask ), 10 );
  }

#if !GENERATINGCFM

pascal void ApplicationWaker::RingAlarmGlue()
  {
	ApplicationWaker *self = static_cast<ApplicationWaker *>( RegisterA1() );
	
	A5 savedA5;
	self->a5.Restore();
	RingAlarm( self );
	savedA5.Restore();
  }

#endif
