// FileOpener.cp

#ifndef FileOpener_h
#include "FileOpener.h"
#endif
#ifndef FileWritingPath_h
#include "FileWritingPath.h"
#endif

void FileOpener::BuildParamBlock( const FSSpec& file,
											 SignedByte permission )
  {
	// ioDirID isn't in ioParam, but ioPermssn
	// isn't in fileParam.  What's up with that?
	
	ioParam.ioVersNum = 0;
	ioParam.ioVRefNum = file.vRefNum;
	fileParam.ioDirID = file.parID;
	ioParam.ioNamePtr = const_cast<StringPtr>( file.name );
	ioParam.ioPermssn = permission;
  }

Task *FileOpener::OpenDataFork( const FSSpec& file,
										  FileReadingPath& thePath,
										  SignedByte permission )
  {
	Assert( !Running() );
	Assert( !thePath.IsOpen() );
	
	path = &thePath;
	dataFork = true;
	BuildParamBlock( file, permission );

	return this;
  }

Task *FileOpener::OpenDataFork( const FSSpec& file,
										  FileWritingPath& thePath,
										  SignedByte permission )
  {
	Assert( !Running() );
	Assert( !thePath.IsOpen() );
	
	path = &thePath;
	dataFork = true;
	BuildParamBlock( file, permission );

	return this;
  }

Task *FileOpener::OpenResourceFork( const FSSpec& file,
												FileReadingPath& thePath,
												SignedByte permission )
  {
	Assert( !Running() );
	Assert( !thePath.IsOpen() );
	
	path = &thePath;
	dataFork = false;
	BuildParamBlock( file, permission );

	return this;
  }

Task *FileOpener::OpenResourceFork( const FSSpec& file,
												FileWritingPath& thePath,
												SignedByte permission )
  {
	Assert( !Running() );
	Assert( !thePath.IsOpen() );
	
	path = &thePath;
	dataFork = false;
	BuildParamBlock( file, permission );

	return this;
  }

void FileOpener::Launch()
  {
	Assert( !Running() );

	if ( dataFork )
		PBHOpenDFAsync( this );
	 else
		PBHOpenRFAsync( this );
  }

void FileOpener::Kill()
  {
  }

void FileOpener::AtCompletion()
  {
	Assert( path != 0 );
	Assert( !path->IsOpen() );
	
	if ( Error() == noErr )
		path->SetRefNum( ioParam.ioRefNum );
  }
