// TeeBase.cp

#ifndef TeeBase_h
#include "TeeBase.h"
#endif
#ifndef PipeOutput_h
#include "PipeOutput.h"
#endif

TeeBase::TeeBase( Data theWholeRing,
						PipeOutputMaster *theOutputs,
						uint32 theOutputCount )
  : wholeRing( theWholeRing ),
 	 input( theWholeRing, *this ),
 	 outputs( theOutputs ),
 	 outputCount( theOutputCount )
  {
	Assert( theOutputCount > 0 );
  }

void TeeBase::InitializeOutputs()
  {
	for ( uint32 i = 0; i < outputCount; i++ )
		outputs[i].Initialize( wholeRing, *this );
	
	Assert( AuditTotalCorrect() );
	Assert( OutputAuditsMatch() );
	Assert( SegmentEndsMatch() );
  }

void TeeBase::Reset()
  {
	input.Reset();
	for ( uint32 i = 0; i < outputCount; i++ )
		outputs[i].Reset();

	Assert( AuditTotalCorrect() );
	Assert( OutputAuditsMatch() );
	Assert( SegmentEndsMatch() );
  }

void TeeBase::PushToOutputs( PipePacket packet )
  {
	for ( uint32 i = 0; i < outputCount; i++ )
		outputs[i].PushIn( packet );
  }

void TeeBase::PushToInput()
  {
	Assert( AuditTotalCorrect() );
	Assert( OutputAuditsMatch() );
	Assert( SegmentEndsMatch() );

	PipePacket packet = outputs[0].Leaving();
	
	for ( uint32 i = 1; i < outputCount; i++ )
		packet.LimitTo( outputs[i].Leaving() );
	
	if ( packet.IsZero() )
		return;
	
	for ( uint32 i = 0; i < outputCount; i++ )
		outputs[i].PullOut( packet );
	
	input.PushIn( packet );

	Assert( AuditTotalCorrect() );
	Assert( OutputAuditsMatch() );
	Assert( SegmentEndsMatch() );
  }

bool TeeBase::OutputAuditsMatch() const
  {
	uint32 total = outputs[0].Audit();
	for ( uint32 i = 0; i < outputCount; i++ )
		if ( total != outputs[i].Audit() )
			return false;
	
	return true;
  }

bool TeeBase::AuditTotalCorrect() const
  {
	return outputs[0].Audit() + input.Audit() == wholeRing.Length();
  }

bool TeeBase::SegmentEndsMatch() const
  {
	for ( uint32 i = 0; i < outputCount; i++ )
	  {
		if ( input.EnteringEnd() != outputs[i].ExitingStart() )
			return false;
		
		if ( input.ExitingStart() != outputs[i].EnteringEnd() )
			return false;
	  }
	
	return true;
  }
