// InputPump.cp

#ifndef InputPump_h
#include "InputPump.h"
#endif
#ifndef InStream_h
#include "InStream.h"
#endif
#ifndef PipeInput_h
#include "PipeInput.h"
#endif

InputPump::InputPump()
  : pipe( 0 ),
	 stream( 0 ),
	 pauseTime( Millisecs( 0 ) ),
	 sequence( *this )
  {
  }

Task *InputPump::operator()( InStream& from,
									  PipeInput& to,
									  WaitLength pause )
  {
	stream = &from;
	pipe = &to;
	pauseTime = pause;
	
	return sequence.Start( Step( 0, &InputPump::Accept ) );
  }

TaskStep<InputPump> InputPump::Accept( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	if ( dying )
		return Step( pipe->Pass() );

	return Step( pipe->Accept( 1 ), &InputPump::Read );
  }

TaskStep<InputPump> InputPump::Read( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	if ( dying )
		return Step( pipe->Pass() );

	readBuffer.Reset( pipe->Buffer().Unused().Head( pipe->ReasonablePassSize() ) );
	
	return Step( stream->ReadSome( readBuffer ), &InputPump::Pass );
  }

TaskStep<InputPump> InputPump::Pass( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	pipe->Buffer().AdvanceMark( readBuffer.UsedLength() );

	if ( dying )
		return Step( pipe->Pass() );

	if ( stream->Ended() || stream->Broken() )
		return Step( pipe->Flush() );
	
	return Step( pipe->Pass(), &InputPump::Pause );
  }

TaskStep<InputPump> InputPump::Pause( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	if ( dying )
		return Step( 0 );
	
	if ( readBuffer.Exhausted() )
		return Step( 0, &InputPump::Accept );

	if ( readBuffer.Fresh() )
	  {
		readBuffer.Reset( pipe->Buffer().Unused().Head( 1 ) );
		return Step( stream->ReadAll( readBuffer ), &InputPump::Wake );
	  }
	
	return Step( wait( pauseTime ), &InputPump::Accept );
  }

TaskStep<InputPump> InputPump::Wake( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	pipe->Buffer().AdvanceMark( readBuffer.UsedLength() );

	if ( dying )
		return Step( pipe->Pass() );

	if ( stream->Ended() || stream->Broken() )
		return Step( pipe->Flush() );
	
	if ( readBuffer.Fresh() )
		return Step( wait( pauseTime ), &InputPump::Accept );
	
	return Step( 0, &InputPump::Accept );
  }
