// ObjectPump.cp

#ifndef ObjectPump_h
#include "ObjectPump.h"
#endif
#ifndef PipeOutput_h
#include "PipeOutput.h"
#endif
#ifndef Pumpable_h
#include "Pumpable.h"
#endif

template < class DeliveryTime >
ObjectPump<DeliveryTime>::ObjectPump()
  : pipe( 0 ),
	 target( 0 ),
	 sequence( *this )
  {
  }

template < class DeliveryTime >
Task *ObjectPump<DeliveryTime>::operator()( PipeOutput& from,
														  Target& to )
  {
	pipe = &from;
	target = &to;
	
	return sequence.Start( Step( 0, &Pump::Accept ) );
  }

template < class DeliveryTime >
ObjectPump<DeliveryTime>::Step ObjectPump<DeliveryTime>::Accept( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( target != 0 );
	
	if ( dying )
		return Step( 0 );
	
	return Step( pipe->Accept( 1 ), &Pump::Deliver );
  }

template < class DeliveryTime >
ObjectPump<DeliveryTime>::Step ObjectPump<DeliveryTime>::Deliver( bool dying, DeliveryTime )
  {
	Assert( pipe != 0 );
	Assert( target != 0 );
	
	if ( dying )
		return Step( 0 );
	
	if ( !pipe->Buffer().Exhausted() )
	  {
		target->ProcessData( pipe->Buffer() );
		pipe->Buffer().AdvanceMark( pipe->Buffer().UnusedLength() );
	  }
	
	if ( pipe->Flushing() )
	  {
		target->FlushData();
		return Step( pipe->AcknowledgeFlush(), &Pump::Accept );
	  }
		
	return Step( pipe->Pass(), &Pump::Accept );
  }

template class ObjectPump<ApplicationTime>;
template class ObjectPump<DeferredTaskTime>;
