// OutputPump.cp

#ifndef OutputPump_h
#include "OutputPump.h"
#endif
#ifndef PipeOutput_h
#include "PipeOutput.h"
#endif
#ifndef OutStream_h
#include "OutStream.h"
#endif

OutputPump::OutputPump()
  : pipe( 0 ),
	 stream( 0 ),
	 sequence( *this )
  {
  }

Task *OutputPump::operator()( PipeOutput& from, OutStream& to )
  {
	pipe = &from;
	stream = &to;
		
	return sequence.Start( Step( 0, &OutputPump::Accept ) );
  }

TaskStep<OutputPump> OutputPump::Accept( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );
	
	if ( dying )
		return Step( 0 );

	if ( pipe->Buffer().Exhausted() && pipe->Flushing() )
		return Step( stream->Flush(), &OutputPump::AcknowledgeFlush );
		
	return Step( pipe->Accept( 1 ), &OutputPump::Write );
  }

TaskStep<OutputPump> OutputPump::Write( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );
	
	if ( dying )
		return Step( 0 );

	writeBuffer.Reset( pipe->Buffer().Unused().Head( pipe->ReasonablePassSize() ) );
	
	return Step( stream->WriteAll( writeBuffer ), &OutputPump::Pass );
  }

TaskStep<OutputPump> OutputPump::Pass( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	pipe->Buffer().AdvanceMark( writeBuffer.UsedLength() );
	
	if ( dying || stream->Broken() )
		return Step( pipe->Pass() );
		
	return Step( pipe->Pass(), &OutputPump::Accept );
  }

TaskStep<OutputPump> OutputPump::AcknowledgeFlush( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );
	Assert( stream != 0 );

	if ( dying || stream->Broken() )
		return Step( 0 );
	
	Assert( pipe->Buffer().Exhausted() );
	Assert( pipe->Flushing() );
	
	return Step( pipe->AcknowledgeFlush(), &OutputPump::Accept );
  }
