// StreamToPipe.cp

#ifndef StreamToPipe_h
#include "StreamToPipe.h"
#endif
#ifndef PipeInput_h
#include "PipeInput.h"
#endif
#ifndef MinMax_h
#include "MinMax.h"
#endif

StreamToPipe::StreamToPipe()
  : pipe( 0 ),
	 sequence( *this )
  {
  }

StreamToPipe::StreamToPipe( PipeInput& thePipe )
  : pipe( &thePipe ),
	 sequence( *this )
  {
  }

Task *StreamToPipe::BlockingWrite( ConstData data )
  {
	Assert( pipe != 0 );
	toWrite.Reset( data );
	
	pipe->Buffer() << toWrite;
	return sequence.Start( Step( pipe->Pass(), &StreamToPipe::BlockingAccept ) );
  }

Task *StreamToPipe::NonblockingWrite( ConstData data )
  {
	Assert( pipe != 0 );
	toWrite.Reset( data );
	
	pipe->Buffer() << toWrite;
	return sequence.Start( Step( pipe->Pass(), &StreamToPipe::NonblockingAccept ) );
  }

Task *StreamToPipe::FlushingTask()
  {
	Assert( pipe != 0 );
	return pipe->Flush();
  }

TaskStep<StreamToPipe> StreamToPipe::BlockingAccept( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );

	if ( dying || toWrite.Exhausted() )
	  {
		ReportWrite( toWrite.UsedLength() );
		return Step( 0 );
	  }
	
	Assert( pipe->Buffer().Exhausted() );
	
	uint32 requestSize = Min( toWrite.UnusedLength(), pipe->ReasonableAcceptSize() );
	return Step( pipe->Accept( requestSize ), &StreamToPipe::BlockingPass );
  }

TaskStep<StreamToPipe> StreamToPipe::BlockingPass( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );

	if ( dying )
	  {
		ReportWrite( toWrite.UsedLength() );
		return Step( 0 );
	  }
	
	pipe->Buffer() << toWrite;
	return Step( pipe->Pass(), &StreamToPipe::BlockingAccept );
  }

TaskStep<StreamToPipe> StreamToPipe::NonblockingAccept( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );

	if ( dying || toWrite.Exhausted() )
	  {
		ReportWrite( toWrite.UsedLength() );
		return Step( 0 );
	  }
	
	Assert( pipe->Buffer().Exhausted() );
	
	return Step( pipe->Accept( 0 ), &StreamToPipe::NonblockingPass );
  }

TaskStep<StreamToPipe> StreamToPipe::NonblockingPass( bool dying, DeferredTaskTime )
  {
	Assert( pipe != 0 );

	if ( dying || pipe->Buffer().Exhausted() )
	  {
		ReportWrite( toWrite.UsedLength() );
		return Step( 0 );
	  }
	
	pipe->Buffer() << toWrite;
	return Step( pipe->Pass(), &StreamToPipe::NonblockingAccept );
  }
