// RaceBase.cp

#ifndef RaceBase_h
#include "RaceBase.h"
#endif

RaceBase::RaceBase()
  : numberToLetFinish( 0 ),
	 racerCount( 0 ),
	 numberFinished( 0 ),
	 launching( false ),
	 killAfterLaunch( false ),
	 racerFinished( this, &RaceBase::RacerFinished )
  {
  }

Task *RaceBase::GetReady( uint32 theNumberToLetFinish,
								  uint32 theRacerCount )
  {
	Assert( !launching );
	Assert( theNumberToLetFinish > 0 );
	Assert( theNumberToLetFinish <= theRacerCount );
	
	numberToLetFinish = theNumberToLetFinish;
	racerCount = theRacerCount;
	numberFinished = 0;
	killAfterLaunch = false;
	
	return this;
  }

void RaceBase::Launch()
  {
	Assert( !launching );
	Assert( numberToLetFinish > 0 );
	Assert( numberToLetFinish <= racerCount );
	Assert( numberFinished == 0 );
	Assert( !killAfterLaunch );

	launching = true;
	LaunchRacers( racerFinished );
	launching = false;
	
	if ( killAfterLaunch )
		Kill();
  }

void RaceBase::Kill()
  {
	Assert( !launching );

	if ( numberFinished < racerCount )
		KillRacers();
  }

void RaceBase::RacerFinished( DeferredTaskTime )
  {
	Assert( numberFinished < racerCount );
	Assert( numberToLetFinish > 0 );
	Assert( numberToLetFinish <= racerCount );
	
	numberFinished++;
		
	if ( numberFinished == racerCount )
		Complete();
	 else
		if ( numberFinished == numberToLetFinish )
		  {
			if ( launching )
				killAfterLaunch = true;
			 else
				KillRacers();
		  }
  }
