// TaskSequencer.cp

#ifndef TaskSequencer_cp
#define TaskSequencer_cp

#ifndef TaskSequencer_h
#include "TaskSequencer.h"
#endif
#ifndef ApplicationTaskQueue_h
#include "ApplicationTaskQueue.h"
#endif

template < class Sequence >
TaskSequencer<Sequence>::TaskSequencer( Sequence& theSequence )
  : sequence( theSequence ),
	 deferredCompletor( this, &TaskSequencer<Sequence>::DeferredCompletor ),
	 applicationCompletor( this, &TaskSequencer<Sequence>::ApplicationCompletor ),
  	 finishingCompletor( this, &TaskSequencer<Sequence>::FinishingCompletor ),
  	 dying( true )
  {
  }

template < class Sequence >
void TaskSequencer<Sequence>::LaunchStep()
  {
	if ( step.IsDeferred() )
		life.Launch( step.Task(), deferredCompletor );
	 else if ( step.IsApplication() )
		life.Launch( step.Task(), applicationCompletor );
	 else
	  {
		if ( step.Task() == 0 )
			Complete();
		 else
			life.Launch( step.Task(), finishingCompletor );
	  }
  }

template < class Sequence >
Task *TaskSequencer<Sequence>::Start( Step nextStep )
  {
	step = nextStep;
	dying = false;
	return this;
  }

template < class Sequence >
void TaskSequencer<Sequence>::Launch()
  {
	Assert( !dying );
	LaunchStep();
  }

template < class Sequence >
void TaskSequencer<Sequence>::Kill()
  {
	if ( !dying )
	  {
		dying = true;
		life.Kill();
	  }
  }

template < class Sequence >
void TaskSequencer<Sequence>::DeferredCompletor( DeferredTaskTime time )
  {
	Assert( step.DeferredMethod() != 0 );

	do
		step = ( sequence.*step.DeferredMethod() )( dying, time );
	 while ( step.Task() == 0 && step.DeferredMethod() != 0 );
	
	LaunchStep();
  }

template < class Sequence >
void TaskSequencer<Sequence>::ApplicationCompletor( ApplicationTime time )
  {
	Assert( step.ApplicationMethod() != 0 );

	do
		step = ( sequence.*step.ApplicationMethod() )( dying, time );
	 while ( step.Task() == 0 && step.ApplicationMethod() != 0 );

	LaunchStep();
  }

template < class Sequence >
void TaskSequencer<Sequence>::FinishingCompletor( DeferredTaskTime )
  {
	Complete();
  }

#endif
