// InStream.cp

#ifndef InStream_h
#include "InStream.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef Buffer_h
#include "Buffer.h"
#endif

InStream::InStream()
  : buffer( 0 ),
	 broken( false ),
	 ended( false )
  {
  }

void InStream::Reset()
  {
	buffer = 0;
	broken = false;
	ended = false;
  }

Task *InStream::ReadSome( Buffer& theBuffer )
  {
	Assert( !Broken() );
	Assert( !Ended() );
	
	if ( theBuffer.UnusedLength() == 0 )
		return 0;
	
	buffer = &theBuffer;
	return NonblockingRead( theBuffer.Unused() );
  }

Task *InStream::ReadAll( Buffer& theBuffer )
  {
	Assert( !Broken() );
	Assert( !Ended() );
	
	if ( theBuffer.UnusedLength() == 0 )
		return 0;
	
	buffer = &theBuffer;
	return BlockingRead( theBuffer.Unused() );
  }

void InStream::ReportRead( uint32 length )
  {
	Assert( buffer != 0 );
	Assert( !Broken() );
	Assert( !Ended() );
	
	buffer->AdvanceMark( length );
	buffer = 0;
  }

void InStream::ReportFailure()
  {
	Assert( !Broken() );
	Assert( !Ended() );
	
	broken = true;
  }

void InStream::ReportEnd()
  {
	Assert( !Broken() );
	Assert( !Ended() );
	
	ended = true;
  }
