// OutStream.cp

#ifndef OutStream_h
#include "OutStream.h"
#endif
#ifndef Assert_h
#include "Assert.h"
#endif
#ifndef ConstBuffer_h
#include "ConstBuffer.h"
#endif

OutStream::OutStream()
  : buffer( 0 ),
	 broken( false )
  {
  }

void OutStream::Reset()
  {
	buffer = 0;
	broken = false;
  }

Task *OutStream::WriteSome( ConstBuffer& theBuffer )
  {
	Assert( !Broken() );
	
	if ( theBuffer.UnusedLength() == 0 )
		return 0;
	
	buffer = &theBuffer;
	return NonblockingWrite( theBuffer.Unused() );
  }

Task *OutStream::WriteAll( ConstBuffer& theBuffer )
  {
	Assert( !Broken() );

	if ( theBuffer.UnusedLength() == 0 )
		return 0;
	
	buffer = &theBuffer;
	return BlockingWrite( theBuffer.Unused() );
  }

Task *OutStream::Flush()
  {
	Assert( !Broken() );
	
	buffer = 0;
	return FlushingTask();
  }

void OutStream::ReportWrite( uint32 length )
  {
	Assert( buffer != 0 );
	Assert( !Broken() );
	
	buffer->AdvanceMark( length );
	buffer = 0;
  }

void OutStream::ReportFailure()
  {
	Assert( !Broken() );
	broken = true;
  }
