//
//	This code snippet does not perform error checking and does not deallocate many of the things
//	it allocates.  If you ship code like that, you're a goon.
//

#include <MacTypes.h>
#include <MacMemory.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <MacWindows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <ToolUtils.h>
#include <SegLoad.h>
#include <Processes.h>
#include <Movies.h>
#include <ImageCodec.h>
#include <QuickTimeComponents.h>
#include <StandardFile.h>
#include <NumberFormatting.h>
#include <Files.h>
#include <Endian.h>
#include <Timer.h>
#include <FixMath.h>



Boolean done = false;
WindowPtr window = nil;

void Initialize(void);
void Draw();
void Run();

void PolyToPolyMatrix(const Fixed *source, const Fixed *dest, MatrixRecord *map);


void main(void)
{
	Initialize();
	
	while( !done ) 
		Run();
}


void Initialize(void)
{
	MenuHandle	mh = nil;
	Rect bounds = {50, 16, 584, 784};

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	// The dialog manager doesn't like it if there are no menus around. 
	mh = NewMenu( 128,"\p" );
	MacInsertMenu( mh, 0 );
	
	
	window = NewCWindow( nil, &bounds, "\pBlitting", true, documentProc, (WindowPtr)-1, false, 0 );
	SetPort( window );
}

static GraphicsImportComponent GetFileAndGrip()
{
	OSType				theTypeListQTIF[1]	= { kQTFileTypeQuickTimeImage };
	StandardFileReply	theReply;
	FSSpec				fileSpec;
	GraphicsImportComponent g = 0;
	OSErr				err;
	
	StandardGetFilePreview(
		nil,
		1,
		theTypeListQTIF,
		&theReply );
	
	if( ! theReply.sfGood )
		return 0;

	fileSpec = theReply.sfFile;
	err = GetGraphicsImporterForFile( &fileSpec, &g );
	return g;	
}

static void RectToPoly( Rect *r, Fixed poly[9] )
{
	poly[0] = 4;
	poly[1] = IntToFixed( r->left );
	poly[2] = IntToFixed( r->top );
	poly[3] = IntToFixed( r->right );
	poly[4] = IntToFixed( r->top );
	poly[5] = IntToFixed( r->right );
	poly[6] = IntToFixed( r->bottom );
	poly[7] = IntToFixed( r->left );
	poly[8] = IntToFixed( r->bottom );
}

static void PerspectiveRectMatrix( MatrixRecord *m, Rect *r1, Rect *r2 )
{
	Fixed fromPoly[9];
	Fixed toPoly[9];
	
	RectToPoly( r1, fromPoly );
	RectToPoly( r2, toPoly );
	
	// Skew the bottom-right corner of the polygon downwards.
	toPoly[6] += IntToFixed( 200 );
	
	PolyToPolyMatrix( fromPoly, toPoly, m );
}

static OSType getEffectType(QTAtomContainer videoFilterSettings)
{
	QTAtom effectTypeAtom;
	OSType effectType = 0;

	if (videoFilterSettings) {
		effectTypeAtom = QTFindChildByIndex(videoFilterSettings, kParentAtomIsContainer, kParameterWhatName, 1, nil);
		if (effectTypeAtom)
			QTCopyAtomDataToPtr(videoFilterSettings, effectTypeAtom, false, sizeof(effectType), &effectType, nil);
	}

	return effectType;
}

enum {
	kCopyBits,
	kDecoSeq,
	kDecoSeqMatrix,
	kDecoSeqFilter,
	kDecoSeqFilterMatrix
};

Rect offscreenBounds = { 0, 0, 256, 256 };
long offscreenDepth = 32;
GWorldPtr gworld = nil;
ImageDescriptionHandle imageDesc = nil;
GraphicsImportComponent gi = nil;
Rect giBounds;

Rect targetBounds = { 16, 444, 16+256, 444+256 };
Point modeText = { 400, 16 };
Point gmText = { 450, 16 };
Point usText = { 500, 16 };

short mode = kCopyBits;
short graphicsMode = ditherCopy;
Boolean reinit = true;
RGBColor lightGray = { 0x8000, 0x8000, 0x8000 };
//RGBColor lightGray = { 0xcccc, 0xcccc, 0xcccc };
PixPatHandle lightGrayPP = 0;

ImageSequence imageSeq = 0;

QTAtomContainer videoFilterList = 0;
QTAtomContainer videoFilter = 0;
ImageSequenceDataSource effectDataSource = 0;

void Draw()
{
	CGrafPtr savegw;
	GDHandle savegd;
	Point m;
	UnsignedWide startus, endus;
	long us;
	Str255 str;
	MatrixRecord matrix;
	
	GetGWorld( &savegw, &savegd );
	
	// got a gworld yet?
	if( !gworld ) {
		if( QTNewGWorld( &gworld, offscreenDepth, &offscreenBounds, nil, nil, kICMTempThenAppMemory ) )
			return;
		LockPixels( gworld->portPixMap );
		MakeImageDescriptionForPixMap( gworld->portPixMap, &imageDesc );
	}
	
	// got the graphics importers?
	if( !gi ) {
		gi = GetFileAndGrip();
		if( !gi )
			ExitToShell();
		GraphicsImportSetGWorld( gi, gworld, nil );
		GraphicsImportGetNaturalBounds( gi, &giBounds );
	}
	
	// rebuild our blitter?
	if( reinit ) {
		if( imageSeq )
			CDSequenceEnd( imageSeq );
		imageSeq = nil;
		
#if 1
		if( videoFilter )
			QTDisposeAtomContainer( videoFilter );
		videoFilter = nil;
#endif
		
		switch( mode ) {
			case kCopyBits:
				// no setup.
				break;
				
			case kDecoSeq:
				RectMatrix(&matrix, &offscreenBounds, &targetBounds);
   
				DecompressSequenceBegin( &imageSeq, imageDesc, (CGrafPtr)window, nil, nil, 
						&matrix, graphicsMode,
						nil, 0, codecNormalQuality, nil );
				break;
				
			case kDecoSeqMatrix:
				PerspectiveRectMatrix(&matrix, &offscreenBounds, &targetBounds);
   
				DecompressSequenceBegin( &imageSeq, imageDesc, (CGrafPtr)window, nil, nil, 
						&matrix, graphicsMode,
						nil, 0, codecNormalQuality, nil );
				break;
				
			case kDecoSeqFilter:
			case kDecoSeqFilterMatrix:
			{
				long aLong;
				OSType effectType;
				ImageDescriptionPtr idp;
				ImageDescriptionHandle effectDesc;
				
				// display filter dialog
				
				QTParameterDialog dialog = nil;

				if( !videoFilter )
					QTNewAtomContainer(&videoFilter);

				// get all the single source filters
				if( !videoFilterList )
					QTGetEffectsList(&videoFilterList, 1, 1, 0);

				if (QTCreateStandardParameterDialog(videoFilterList, videoFilter, pdOptionsCollectOneValue, &dialog) == noErr) {
					Boolean done = false;
					OSErr dialogResult;

					while (!done) {
						EventRecord theEvent;

						WaitNextEvent( everyEvent, &theEvent, 0, nil );
						dialogResult = QTIsStandardParameterDialogEvent (&theEvent, dialog);
						if ((dialogResult == codecParameterDialogConfirm) || (dialogResult == userCanceledErr))
							done = true;

						if (dialogResult == featureUnsupported)
						{
							dialogResult = noErr;
						
							switch (theEvent.what)
							{
								case updateEvt:
									BeginUpdate((WindowPtr) theEvent.message);
									EndUpdate((WindowPtr) theEvent.message);
									break;
							}
						}
					}

					QTDismissStandardParameterDialog(dialog);
				}

				effectType = getEffectType(videoFilter);
				effectType = EndianU32_BtoN(effectType);

				// add the source to the effect description
				aLong = EndianU32_BtoN(OSTypeConst('srca'));
				QTInsertChild(videoFilter, kParentAtomIsContainer, kEffectSourceName, 1, 0,  sizeof(aLong), &aLong, nil);

				HLockHi(videoFilter);

				// make an effects sample description
				effectDesc = (ImageDescriptionHandle)NewHandleClear(sizeof(ImageDescription));

				idp = *effectDesc;
				idp->idSize = sizeof(ImageDescription);
				idp->cType = effectType;
				idp->width = offscreenBounds.right - offscreenBounds.left;
				idp->height = offscreenBounds.bottom - offscreenBounds.top;
				idp->hRes = 72L<<16;
				idp->vRes = 72L<<16;
				idp->depth = 24;
				idp->frameCount = 1;
				idp->clutID = -1;

				// make the effect sequence
				if( kDecoSeqFilter == mode )
					RectMatrix(&matrix, &offscreenBounds, &targetBounds);
				else
					PerspectiveRectMatrix(&matrix, &offscreenBounds, &targetBounds);

				DecompressSequenceBeginS( &imageSeq, effectDesc, StripAddress(*videoFilter),
						GetHandleSize(videoFilter), (CGrafPtr)window, nil, nil, 
						&matrix, graphicsMode,
						nil, 0, codecNormalQuality, nil);

				// add the data source
				CDSequenceNewDataSource(imageSeq, &effectDataSource, OSTypeConst('srca'), 1,
							(Handle)imageDesc, nil, 0);
			}	break;
		}
		
		//EraseRect( &window->portRect );
		if( !lightGrayPP ) {
			lightGrayPP = NewPixPat();
			MakeRGBPat( lightGrayPP, &lightGray );
		}
		FillCRect( &window->portRect, lightGrayPP );
		reinit = false;
	}
	
	// redraw the offscreen using the mouse
	GetMouse( &m );
	SetGWorld( gworld, nil );
	EraseRect( &offscreenBounds );
	{
		Rect b1;
		Rect b2;
		b1.top = m.v;
		b1.left = m.h;
		b1.bottom = b1.top + giBounds.bottom;
		b1.right = b1.left + giBounds.right;
		b2.top = offscreenBounds.bottom - b1.bottom;
		b2.left = offscreenBounds.right - b1.right;
		b2.bottom = offscreenBounds.bottom - b1.top;
		b2.right = offscreenBounds.right - b1.left;
		
		GraphicsImportSetBoundsRect( gi, &b2 );
		GraphicsImportDraw(gi);
		GraphicsImportSetBoundsRect( gi, &b1 );
		GraphicsImportDraw(gi);
	}
	
	SetGWorld( savegw, savegd );
	
	// blit.  count the microseconds.
	Microseconds(&startus);
	
	switch( mode ) {
		case kCopyBits:
			CopyBits( &((GrafPtr)gworld)->portBits,
					  &((GrafPtr)window)->portBits,
					  &offscreenBounds, 
					  &targetBounds,
					  graphicsMode,
					  nil );
			break;
		case kDecoSeq:
		case kDecoSeqMatrix:
			DecompressSequenceFrameS(imageSeq, GetPixBaseAddr(gworld->portPixMap),
					(**imageDesc).dataSize, 0, nil, nil);			
			break;
			
		case kDecoSeqFilter:
		case kDecoSeqFilterMatrix:
			CDSequenceSetSourceData(effectDataSource, GetPixBaseAddr(gworld->portPixMap),
					(**imageDesc).dataSize );
			DecompressSequenceFrameS(imageSeq, StripAddress(*videoFilter), 
					GetHandleSize(videoFilter), 0, nil, nil);
			break;
	}
	
	Microseconds(&endus);
	CompSub( (wide*)&startus, (wide*)&endus );
	us = endus.lo;

	// draw the method
	TextFont( kFontIDTimes );
	TextSize( 30 );
	TextMode( srcCopy );
	MoveTo( modeText.h, modeText.v );
	switch( mode ) {
		case kCopyBits:
			DrawString( "\pCopyBits" );
			break;
		case kDecoSeq:
			DrawString( "\pDecompressSequenceFrameS" );
			break;
		case kDecoSeqMatrix:
			DrawString( "\pDecompressSequenceFrameS with matrix" );
			break;
		case kDecoSeqFilter:
			DrawString( "\pDecompressSequenceFrameS with filter" );
			break;
		case kDecoSeqFilterMatrix:
			DrawString( "\pDecompressSequenceFrameS with filter and matrix" );
			break;
	}
	
	// draw the graphics mode
	MoveTo( gmText.h, gmText.v );
	switch( graphicsMode ) {
		case srcCopy:
			DrawString( "\psrcCopy" );
			break;
		case srcCopy+ditherCopy:
			DrawString( "\psrcCopy + ditherCopy" );
			break;
	}
	
	// draw the microseconds
	MoveTo( usText.h, usText.v );
	NumToString( us, str );
	DrawString( str );
	DrawString( "\p sec    " );
}

void Run()
{
	char ch;
	EventRecord event;
	
	if( WaitNextEvent( everyEvent, &event, 0, nil ) ) {
		switch( event.what ) {
			case mouseDown:
				done = true;
				break;
				
			case keyDown:
				ch = event.message & charCodeMask;
				switch( ch ) {
					case 'q':
					case 'Q':
					case 27:
						done = true;
						break;
					
					case '1':
						mode = kCopyBits;
						reinit = true;
						break;
					
					case '2':
						mode = kDecoSeq;
						reinit = true;
						break;
					
					case '3':
						mode = kDecoSeqMatrix;
						reinit = true;
						break;
					
					case '4':
						mode = kDecoSeqFilter;
						reinit = true;
						break;
					
					case '5':
						mode = kDecoSeqFilterMatrix;
						reinit = true;
						break;
					
					case 'f':
					case 'F':
						if( gi )
							CloseComponent( gi );
						gi = nil;
						break;
						
					case 'd':
					case 'D':
						if( graphicsMode == srcCopy+ditherCopy )
							graphicsMode = srcCopy;
						else
							graphicsMode = srcCopy+ditherCopy;
						reinit = true;
						break;
				}
				break;
				
			case updateEvt:
				BeginUpdate(window);
				EndUpdate(window);
				break;
		}
	} else {
		Draw();
	}
}

//====================================================================================

/* ******************************** PolyToPolyMatrix ******************************** */

// PolyToPolyMatrix takes two polygons and generates a matrix mapping from one to the other.
// The two polygons are stored as arrays of longs; the first long is the number of points
// in the polygon; the points follow as FixedPoints: x, then y.
// Both polygons should have the same number of points.  
// Polygons with more than 4 points will be treated as though they had only 4 points.

static long MultiplyDivide(long source, long multiplier, long divisor)
{
	wide temp;
	CompMul(source, multiplier, &temp);
	return CompDiv(&temp, divisor, 0);
}

// Hmm, name collision with GXMath.h.  Alas.
#define VectorMultiply MyVectorMultiply

static wide *VectorMultiply(register long count, register const long *vector1, register long step1,
			register const long *vector2, register long step2, register wide *dot)
{
	short flags = count >= 0;
	dot->hi = dot->lo = 0;
	if (!flags)
		count = -count;
	while (count--)
	{	wide temp;
		if (flags ^= 2)
		{
			CompMul(*vector1, *vector2, &temp);
			CompAdd(&temp, dot);
		}
		else
		{
			CompMul(*vector1, *vector2, &temp);
			CompSub(&temp, dot);
		}
		vector1 += step1;
		vector2 += step2;
	}
	return dot;
}

static long VectorMultiplyDivide(long count, const long *vector1, long step1, const long *vector2, long step2, long scalar)
{
	wide temp;
	return CompDiv(VectorMultiply(count, vector1, step1, vector2, step2, &temp), scalar, 0);
}

// ------------------------------------------------------------------------------------------
static unsigned long MyWideSquareRoot(const wide *src)
{
	{	register unsigned long srchi=src->hi;
		register unsigned long srclo=src->lo;
		register unsigned long root=0;
		register unsigned long left=0;
		register unsigned long delta=0;
		unsigned short i, lefthi, deltahi;
	
		i = 16;						/* compute the first 16 bits */
		while (i--)
		{	left = left << 2 | srchi >> 30;
			srchi <<= 2;
			root <<= 1;
			delta = (root << 1) + 1;
			if (left >= delta)
			{	left -= delta;
				root += 1;
			}
		}
	
		i = 14;						/* compute the next 14 bits */
		while(i--)
		{	left = left << 2 | srclo >> 30;
			srclo <<= 2;
			root <<= 1;
			delta = (root << 1) + 1;
			if (left >= delta)
			{	left -= delta;
				root += 1;
			}
		}
	
		i = 2;						/* compute the last two bits */
		while (i--)
		{	lefthi = left >> 30;
			left = left << 2 | srclo >> 30;
			srclo <<= 2;
			root <<= 1;
			deltahi = root >> 31;
			delta = (root << 1) + 1;
			if (lefthi > deltahi || lefthi == deltahi && left >= delta)
			{	if (left < delta)
					lefthi -= deltahi + 1;
				else
					lefthi -= deltahi;
				left -= delta;
				root += 1;
			}
		}
	
		if (lefthi || left > root)			/* round */
			if (root + 1)				/* don't exceed 32 bits */
				root += 1;
	
		return root;
	}
}

static unsigned long Magnitude(long deltaX, long deltaY)
{
	wide temp1, temp2;

	if (deltaX == 0)
		return deltaY < 0 ? -deltaY : deltaY;
	if (deltaY == 0)
		return deltaX < 0 ? -deltaX : deltaX;
	CompMul(deltaX, deltaX, &temp1);
	CompMul(deltaY, deltaY, &temp2);
	CompAdd(&temp1, &temp2);
	return MyWideSquareRoot(&temp2);
}

static void PolyToPoint(const Fixed *poly, FixedPoint *pt)
{
    register Fixed x = fixed1, y = fixed1;
    FixedPoint pt1, pt2;

    if (poly[0] > 1)
    {   pt1.x = poly[3] - poly[1];
        pt1.y = poly[4] - poly[2];
        y = Magnitude(pt1.x, pt1.y);
        switch (poly[0])
        {   case 2:
            break;
            case 3:
                pt2.x = poly[2] - poly[6];
                pt2.y = poly[5] - poly[1];
            goto calcX;
            default:
                pt2.x = poly[2] - poly[8];
                pt2.y = poly[7] - poly[1];
            calcX:
                x = VectorMultiplyDivide(2, &pt1.x, 1, &pt2.x, 1, y);
            break;
        }
    }
    pt->x = x;
    pt->y = y;
}

static void Map1Pt(const FixedPoint *source, Fixed *dest)
{
    register Fixed *destPtr = dest;
    register const FixedPoint *srcPt = source;
    *destPtr++ = fixed1;
    *destPtr++ = 0;
    *destPtr++ = 0;
    *destPtr++ = 0;
    *destPtr++ = fixed1;
    *destPtr++ = 0;
    *destPtr++ = srcPt[0].x;
    *destPtr++ = srcPt[0].y;
    *destPtr = fract1;
}

static void Map2Pt(const FixedPoint *source, Fixed *dest, Fixed scale)
{
    register Fixed *destPtr = dest;
    register const FixedPoint *srcPt = source;
    *destPtr++ = FixDiv(srcPt[1].y - srcPt[0].y, scale);
    *destPtr++ = FixDiv(srcPt[0].x - srcPt[1].x, scale);
    *destPtr++ = 0;
    *destPtr++ = FixDiv(srcPt[1].x - srcPt[0].x, scale);
    *destPtr++ = FixDiv(srcPt[1].y - srcPt[0].y, scale);
    *destPtr++ = 0;
    *destPtr++ = srcPt[0].x;
    *destPtr++ = srcPt[0].y;
    *destPtr = fract1;
}

static void Map3Pt(const FixedPoint *source, Fixed *dest, Fixed scaleX, Fixed scaleY)
{
    register Fixed *destPtr = dest;
    register const FixedPoint *srcPt = source;
    *destPtr++ = FixDiv(srcPt[2].x - srcPt[0].x, scaleX);
    *destPtr++ = FixDiv(srcPt[2].y - srcPt[0].y, scaleX);
    *destPtr++ = 0;
    *destPtr++ = FixDiv(srcPt[1].x - srcPt[0].x, scaleY);
    *destPtr++ = FixDiv(srcPt[1].y - srcPt[0].y, scaleY);
    *destPtr++ = 0;
    *destPtr++ = srcPt[0].x;
    *destPtr++ = srcPt[0].y;
    *destPtr = fract1;
}

static void Map4Pt(const FixedPoint *source, Fixed *dest, Fixed scaleX, Fixed scaleY)
{
    register Fixed *destPtr = dest;
    register const FixedPoint *srcPt = source;
    Fract a1, a2;
    Fixed x0, y0, x1, y1, x2, y2;

    x0 = srcPt[2].x - srcPt[0].x;
    y0 = srcPt[2].y - srcPt[0].y;
    x1 = srcPt[2].x - srcPt[1].x;
    y1 = srcPt[2].y - srcPt[1].y;
    x2 = srcPt[2].x - srcPt[3].x;
    y2 = srcPt[2].y - srcPt[3].y;

    // check if abs(x2) > abs(y2) 
    if ( x2 > 0 ? y2 > 0 ? x2 > y2 : x2 > -y2 : y2 > 0 ? -x2 > y2 : x2 < y2)
        a1 = FracDiv(MultiplyDivide(x0 - x1, y2, x2) - y0 + y1, MultiplyDivide(x1, y2, x2) - y1);
    else
        a1 = FracDiv(x0 - x1 - MultiplyDivide(y0 - y1, x2, y2), x1 - MultiplyDivide(y1, x2, y2));

    // check if abs(x1) > abs(y1) 
    if ( x1 > 0 ? y1 > 0 ? x1 > y1 : x1 > -y1 : y1 > 0 ? -x1 > y1 : x1 < y1)
        a2 = FracDiv(y0 - y2 - MultiplyDivide(x0 - x2, y1, x1), y2 - MultiplyDivide(x2, y1, x1));
    else
        a2 = FracDiv(MultiplyDivide(y0 - y2, x1, y1) - x0 + x2, MultiplyDivide(y2, x1, y1) - x2);

    *destPtr++ = FixDiv(FracMul(a2, srcPt[3].x) + srcPt[3].x - srcPt[0].x, scaleX);
    *destPtr++ = FixDiv(FracMul(a2, srcPt[3].y) + srcPt[3].y - srcPt[0].y, scaleX);
    *destPtr++ = FixDiv(a2, scaleX);
    *destPtr++ = FixDiv(FracMul(a1, srcPt[1].x) + srcPt[1].x - srcPt[0].x, scaleY);
    *destPtr++ = FixDiv(FracMul(a1, srcPt[1].y) + srcPt[1].y - srcPt[0].y, scaleY);
    *destPtr++ = FixDiv(a1, scaleY);
    *destPtr++ = srcPt[0].x;
    *destPtr++ = srcPt[0].y;
    *destPtr = fract1;
}

void PolyToPolyMatrix(const Fixed *source, const Fixed *dest, MatrixRecord *map)
{
    FixedPoint tempPt;
    MatrixRecord tempMap;
    long vectors = source[0];

    PolyToPoint(source, &tempPt);
    switch (vectors)
    {   case 0:
            SetIdentityMatrix(map);
            break;
        case 1:
            Map1Pt((FixedPoint*)&source[1], &tempMap.matrix[0][0]);
            InverseMatrix(&tempMap, map);
            Map1Pt((FixedPoint*)&dest[1], &tempMap.matrix[0][0]);
            goto mapMap;
        case 2:
            Map2Pt((FixedPoint*)&source[1], &tempMap.matrix[0][0], tempPt.y);
            InverseMatrix(&tempMap, map);
            Map2Pt((FixedPoint*)&dest[1], &tempMap.matrix[0][0], tempPt.y);
            goto mapMap;
        case 3:
            Map3Pt((FixedPoint*)&source[1], &tempMap.matrix[0][0], tempPt.x, tempPt.y);
            InverseMatrix(&tempMap, map);
            Map3Pt((FixedPoint*)&dest[1], &tempMap.matrix[0][0], tempPt.x, tempPt.y);
            goto mapMap;
        default:
            Map4Pt((FixedPoint*)&source[1], &tempMap.matrix[0][0], tempPt.x, tempPt.y);
            InverseMatrix(&tempMap, map);
            Map4Pt((FixedPoint*)&dest[1], &tempMap.matrix[0][0], tempPt.x, tempPt.y);
        mapMap:
        	ConcatMatrix(&tempMap, map);
            break;
    }
}

/* ******************************************************************************* */

//====================================================================================
