#include <MacTypes.h>
#include <MacMemory.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <MacWindows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <ToolUtils.h>
#include <SegLoad.h>
#include <Processes.h>
#include <Movies.h>
#include <QuickTimeComponents.h>
#include <StandardFile.h>
#include <NumberFormatting.h>
#include <Files.h>
#include <Endian.h>

	
void Initialize(void);
void CompressSomething( FSSpec *inFile, FSSpec *outFile, short outScript );
void DoCompress(void);


void main(void)
{
	Initialize();
	
	DoCompress();
}


void Initialize(void)
{
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
}

void CompressSomething( FSSpec *inFile, FSSpec *outFile, short outScript )
{
	OSErr err = 0;
	DataCompressorComponent ci = 0;
	short fref = 0;
	long uncompressedLength = 0;
	Handle uncompressedH = nil;
	long compressedLength = 0;
	Handle compressedH = nil;
	unsigned long ignoreThis = 0;
	long count;
	
	err = FSpOpenDF( inFile, fsRdPerm, &fref );
	if( err ) goto bail;
	
	err = GetEOF( fref, &uncompressedLength );
	if( err ) goto bail;
	
	uncompressedH = NewHandle( uncompressedLength );
	err = MemError();
	if( err ) goto bail;
	HLock( uncompressedH );
	
	count = uncompressedLength;
	err = FSRead( fref, &count, *uncompressedH );
	if( err ) goto bail;
	
	FSClose( fref );
	fref = 0;
	
	err = FSpDelete( outFile );
	if( err == fnfErr ) err = noErr;
	if( err ) goto bail;
	
	err = FSpCreate( outFile, 'ZOT ', 'ZOT ', outScript );
	if( err ) goto bail;
	
	err = FSpOpenDF( outFile, fsWrPerm, &fref );
	if( err ) goto bail;
	
	err = OpenADefaultComponent( DataCompressorComponentType, zlibDataCompressorSubType, &ci );
	if( err ) goto bail;
	
	err = DataCodecGetCompressBufferSize( ci, uncompressedLength, (UInt32*) &compressedLength );
	if( err ) goto bail;
	
	compressedH = NewHandle( compressedLength );
	err = MemError();
	if( err ) goto bail;
	HLock( compressedH );
	
	err = DataCodecCompress( ci, 
			*uncompressedH, uncompressedLength, 
			*compressedH, compressedLength, (UInt32*) &compressedLength, &ignoreThis );
	if( err ) goto bail;
	
	// Store the uncompressed size of the compressed data first.
	uncompressedLength = EndianS32_NtoB( uncompressedLength );
	count = 4;
	err = FSWrite( fref, &count, &uncompressedLength );
	if( err ) goto bail;
	
	count = compressedLength;
	err = FSWrite( fref, &count, *compressedH );
	if( err ) goto bail;
	
bail:
	if( fref )
		FSClose( fref );
	if( ci )
		CloseComponent( ci );
	if( uncompressedH )
		DisposeHandle( uncompressedH );
	if( compressedH )
		DisposeHandle( compressedH );
	
	if( err )
		DebugStr( "\p oops." );
}


void DoCompress(void)
{
	StandardFileReply	inreply, outreply;
	OSType				list[1] = { 'TEXT' };
	
	StandardGetFile( nil, 1, list, &inreply );
	if( !inreply.sfGood ) return;
	
	StandardPutFile( "\pSave compressed file as:", "\puntitled.zot", &outreply );
	if( !outreply.sfGood ) return;
	
	CompressSomething( &inreply.sfFile, &outreply.sfFile, outreply.sfScript );
}
